<?php

/**
 * Diafan.CMS
 *
 * @author     diafan.ru
 * @version    5.0
 * @license http://cms.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2012 OOO "Диафан". (http://diafan.ru)
 */
if ( ! defined('DIAFAN'))
{
    include dirname(dirname(dirname(__FILE__))) . '/includes/404.php';
}

/**
 * Shop_model
 *
 * Модель модуля "Магазин"
 */
class Shop_model extends Model
{
    /**
     * @var array значения параметров, влияющих на цену
     */
    private $depends;

    /**
     * @var integer сумма всех покупок текущего пользователя
     */
    private $order_summ = 0;

    /**
     * Конструктор класса
     * 
     * @return void
     */
    public function __construct(&$diafan)
    {
        $this->diafan = &$diafan;

		if($this->diafan->_user->id)
		{
			$this->order_summ = DB::query_result("SELECT SUM(summ) FROM {shop_order} WHERE user_id=%d AND status<>'1'", $this->diafan->_user->id);
		}

        $this->sort_config = $this->expand_sort_with_params();

        if ($this->diafan->sort > count($this->sort_config['sort_directions']))
        {
            include ABSOLUTE_PATH . 'includes/404.php';
        }
    }

    /**
     * Генерирует данные для списка товаров, если отключена группировка товаров
     * 
     * @return boolean
     */
    public function list_()
    {
        if ($this->diafan->cat)
        {
            include ABSOLUTE_PATH . 'includes/404.php';
        }
        $cache_meta = array(
            "name" => "list",
            "language" => _LANG,
            "page" => $this->diafan->page > 1 ? $this->diafan->page : 1,
            "site_id" => $this->diafan->cid,
            "sort" => $this->diafan->sort,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => $this->get_tag_cache_discounts()
        );
        //кеширование
        if ( ! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
        {
            ////navigation//
            $this->diafan->_paginator->page = $this->diafan->page;
            $this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
            $this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
            $this->diafan->_paginator->nen = DB::query_result(
                            "SELECT COUNT(DISTINCT e.id) FROM {shop} AS e"
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                            . " WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
                            . " AND (e.access='0'"
                            . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                            . ")", $this->diafan->cid
            );
            $links = $this->diafan->_paginator->get();
            ////navigation///

            $result = DB::query_range(
                            "SELECT s.id, s.[name], s.[anons], s.timeedit, s.cat_id, s.site_id, s.no_buy, s.article, s.hit, s.new, s.action, s.is_file FROM {shop} AS s"
							. ($this->diafan->sort == 1 || $this->diafan->sort == 2 ?
								" INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
								." AND (pr.date_start >=".time()." OR pr.date_start=0) AND pr.date_finish <=".time()
								." AND pr.currency_id=0"
								." AND pr.role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
								." AND pr.user_id".($this->diafan->_user->id ? " IN (0,".$this->diafan->_user->id.")" : "=0")
								." AND pr.threshold <= ".$this->order_summ : '')
                            . ($this->sort_config['use_params_for_sort'] ? " LEFT JOIN {shop_param_element} AS sp ON sp.element_id=s.id AND sp.trash='0' AND sp.param_id=" . $this->sort_config['param_ids'][$this->diafan->sort] : '')
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='shop'" : "")
                            . " WHERE s.[act]='1' AND s.trash='0' AND s.site_id=%d"
                            . " AND (s.access='0'"
                            . ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                            . ")"
                            . " GROUP BY s.id ORDER BY " . ($this->diafan->sort ? $this->sort_config['sort_directions'][$this->diafan->sort] . ',' : '')
                            . "s.sort ASC, s.id ASC", $this->diafan->cid, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
            );

            $this->result["rows"] = $this->get_elements($result);

            $this->result["paginator"] = $links;

            //сохранение кеша
            $this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
        }

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
                $this->select_price($this->result["rows"][$i]);

                $this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

                $this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
            }
        }
		if($this->diafan->configmodules("theme_list"))
		{
			$this->result["theme"] = $this->diafan->configmodules("theme_list");
		}
		if($this->diafan->configmodules("view_list"))
		{
			$this->result["view"] = $this->diafan->configmodules("view_list");
		}
		else
		{
			$this->result["view"] = 'list';
		}

        $this->result["link_sort"] = $this->get_sort_links($this->sort_config['sort_directions']);

        $this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);

		$this->result['shop_link']=$this->diafan->_route->module($this->diafan->module,true);

        $this->result['sort_config'] = $this->sort_config;

		return true;
    }

    /**
     * Генерирует данные для списка товаров, найденных при помощи поиска
     * 
     * @return boolean
     */
    public function list_search()
    {
        $where_param = '';
        $where = '';
        $values = array();
        $getnav = '';
		$group = '';
        $this->get_where($where, $where_param, $values, $getnav, $group);

        ////navigation//
        $this->diafan->_paginator->page = $this->diafan->page;
        $this->diafan->_paginator->get_nav = $getnav;
        $this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");

        $this->diafan->_paginator->nen = DB::query_result("SELECT COUNT(DISTINCT s.id) FROM {shop} as s "
                        . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='shop'" : "")
                        . $where_param
                        . " WHERE s.[act]='1' AND s.trash='0'"
                        . " AND (s.access='0'"
                        . ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                        . ")"
                        . $where. " GROUP BY s.id".$group, $values);
        $links = $this->diafan->_paginator->get();
        ////navigation///

        $this->result["path"] = $this->get_path('shop');
        $this->result["titlemodule"] = _LANG_SEARCH_GOODS;
        if ( ! $this->diafan->_paginator->nen)
        {
            $this->result["err"] = _LANG_IT_IS_NOT_FOUND;
            return $this->result;
        }

        $result = DB::query_range("SELECT s.id, s.[name], s.timeedit, s.[anons], s.cat_id, s.site_id, s.no_buy, s.article,"
                        . "s.hit, s.new, s.action, s.is_file FROM {shop} AS s"
                        . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='shop'" : "")
						. (($this->diafan->sort == 1 || $this->diafan->sort == 2) && empty($_GET["pr1"]) && empty($_GET["pr2"]) ?
							" INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
							." AND (pr.date_start >=".time()." OR pr.date_start=0) AND pr.date_finish <=".time()
							." AND pr.currency_id=0"
							." AND pr.role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
							." AND pr.user_id".($this->diafan->_user->id ? " IN (0,".$this->diafan->_user->id.")" : "=0")
							." AND pr.threshold <= ".$this->order_summ : '')
                        . $where_param
                        . ($this->sort_config['use_params_for_sort'] ? " LEFT JOIN {shop_param_element} AS sp  ON sp.element_id=s.id AND sp.trash='0' AND sp.param_id=" . $this->sort_config['param_ids'][$this->diafan->sort] : '')
                        . " WHERE s.[act]='1' AND s.trash='0'"
                        . " AND (s.access='0'"
                        . ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                        . ")"
                        . $where
                        . " GROUP BY s.id".$group
                        . " ORDER BY " . ($this->diafan->sort ? $this->sort_config['sort_directions'][$this->diafan->sort] . ',' : '')
                        . " sort ASC, id ASC", $values, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);

        $this->result["rows"] = $this->get_elements($result);

        $this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $links);

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
                $this->select_price($this->result["rows"][$i]);

                $this->useradmin($this->result["rows"][$i]);

                $this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

                $this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
            }
        }
		if($this->diafan->configmodules("theme_list_search"))
		{
			$this->result["theme"] = $this->diafan->configmodules("theme_list_search");
		}
		if($this->diafan->configmodules("view_list_search"))
		{
			$this->result["view"] = $this->diafan->configmodules("view_list_search");
		}
		else
		{
			$this->result["view"] = 'list';
		}

        $this->result["link_sort"] = $this->get_sort_links($this->sort_config['sort_directions']);

        $this->result['sort_config'] = $this->sort_config;

		$this->result['shop_link']=$this->diafan->_route->module($this->diafan->module,true);

        return true;
    }

    /**
     * Генерирует данные для списка товаров, соответствующих значению доп. характеристики
     * 
     * @return boolean
     */
    public function list_param()
    {
        $cache_meta = array(
            "name" => "list_param",
            "language" => _LANG,
            "page" => $this->diafan->page > 1 ? $this->diafan->page : 1,
            "site_id" => $this->diafan->cid,
            "sort" => $this->diafan->sort,
            "param" => $this->diafan->param,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => $this->get_tag_cache_discounts()
        );
        //кеширование
        if ( ! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
        {
            $param_value = DB::fetch_array(DB::query("SELECT param_id, [name] FROM {shop_param_select} WHERE id=%d", $this->diafan->param));
            if ( ! $param_value = DB::fetch_array(DB::query("SELECT param_id, [name] FROM {shop_param_select} WHERE id=%d", $this->diafan->param)))
            {
                include ABSOLUTE_PATH . 'includes/404.php';
            }
            if ( ! $param = DB::fetch_array(DB::query("SELECT [name] FROM {shop_param} WHERE id=%d", $param_value["param_id"])))
            {
                include ABSOLUTE_PATH . 'includes/404.php';
            }
            ////navigation//
            $this->diafan->_paginator->page = $this->diafan->page;
            $this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");

            $this->diafan->_paginator->nen =
                    DB::query_result("SELECT COUNT(DISTINCT s.id) FROM {shop} AS s "
                            . " INNER JOIN {shop_param_element} AS e ON e.element_id=s.id"
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='shop'" : "")
                            . " WHERE s.[act]='1' AND s.trash='0' AND e.param_id=%d AND e.value=%d", $param_value["param_id"], $this->diafan->param
            );
            $this->result["paginator"] = $this->diafan->_paginator->get();
            ////navigation///

            $this->result["path"] = $this->get_path('shop');
            $this->result["titlemodule"] = $param["name"] . ': ' . $param_value["name"];

            $result = DB::query_range(
					"SELECT s.id, s.[name], s.timeedit, s.[anons], s.cat_id, s.site_id, s.no_buy,  s.article, s.is_file FROM {shop} AS s"
					. ($this->diafan->sort == 1 || $this->diafan->sort == 2 ?
						" INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
						." AND (pr.date_start >=".time()." OR pr.date_start=0) AND pr.date_finish <=".time()
						." AND pr.currency_id=0"
						." AND pr.role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
						." AND pr.user_id".($this->diafan->_user->id ? " IN (0,".$this->diafan->_user->id.")" : "=0")
						." AND pr.threshold <= ".$this->order_summ : '')
					. " INNER JOIN {shop_param_element} as e ON e.element_id=s.id"
					. ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='shop'" : "")
					. ($this->sort_config['use_params_for_sort'] ? " LEFT JOIN {shop_param_element} AS sp  ON sp.element_id=s.id AND sp.trash='0' 
								AND sp.param_id=" . $this->sort_config['param_ids'][$this->diafan->sort] : '')
					. " WHERE s.[act]='1' AND s.trash='0' AND e.param_id=%d AND e.value=%d"
					. " AND (s.access='0'"
					. ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
					. ")"
					. " GROUP BY s.id"
					. " ORDER BY " . ($this->diafan->sort ? $this->sort_config['sort_directions'][$this->diafan->sort] . ',' : '')
					. " sort ASC, id ASC", $param_value["param_id"], $this->diafan->param, $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr);

            $this->result["rows"] = $this->get_elements($result);

            //сохранение кеша
            $this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
        }

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
                $this->select_price($this->result["rows"][$i]);

                $this->useradmin($this->result["rows"][$i]);

                $this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

                $this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
            }
        }
		if($this->diafan->configmodules("theme_list_param"))
		{
			$this->result["theme"] = $this->diafan->configmodules("theme_list_param");
		}
		if($this->diafan->configmodules("view_list_param"))
		{
			$this->result["view"] = $this->diafan->configmodules("view_list_param");
		}
		else
		{
			$this->result["view"] = 'list';
		}

        $this->result["link_sort"] = $this->get_sort_links($this->sort_config['sort_directions']);

        $this->result['sort_config'] = $this->sort_config;

		$this->result['shop_link']=$this->diafan->_route->module($this->diafan->module,true);

        $this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
        return true;
    }

    /**
     * Генерирует данные для первой страницы магазина
     * 
     * @return boolean
     */
    public function first_page()
    {
        if ($this->diafan->page)
        {
            include ABSOLUTE_PATH . 'includes/404.php';
        }
        //кеширование
        $cache_meta = array(
            "name" => "first_page",
            "language" => _LANG,
            "site_id" => $this->diafan->cid,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => $this->get_tag_cache_discounts()
        );
        if ( ! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
        {
            $result = DB::query(
                            "SELECT c.id, c.[name], c.[anons], c.timeedit, c.site_id FROM {shop_category} AS c"
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='shop'" : "")
                            . " WHERE c.[act]='1' AND c.parent_id=0 AND c.trash='0' AND c.site_id=%d"
                            . " AND (c.access='0'"
                            . ($this->diafan->_user->id ? " OR c.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                            . ")"
                            . " GROUP BY c.id ORDER by c.sort ASC, c.id ASC", $this->diafan->cid
            );
            while ($row = DB::fetch_array($result))
            {
                $this->result["categories"][$row["id"]]["name"] = $row["name"];
                $this->result["categories"][$row["id"]]["anons"] = $row["anons"];

                if (empty($this->result["timeedit"]) || $row["timeedit"] > $this->result["timeedit"])
                {
                    $this->result["timeedit"] = $row["timeedit"];
                }

                $this->result["categories"][$row["id"]]["children"] = array();
                $result_children = DB::query(
                                "SELECT c.id, c.[name], c.[anons], c.site_id FROM {shop_category} AS c"
                                . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='shop'" : "")
                                . " WHERE c.[act]='1' AND c.parent_id=%d AND c.trash='0' AND c.site_id=%d"
                                . " AND (c.access='0'"
                                . ($this->diafan->_user->id ? " OR c.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                . ")"
                                . " GROUP BY c.id ORDER BY c.sort ASC, c.id ASC", $row["id"], $this->diafan->cid);

                while ($child = DB::fetch_array($result_children))
                {
                    $child["link"] = $this->diafan->_route->link($child["site_id"], 'shop', $child["id"]);
                    if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
                    {
                        $child["img"] = $this->diafan->_images->get($child["id"], $child["name"], $this->diafan->module, $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0, $child["link"], true);
                    }
                    unset($child["site_id"]);
                    $this->result["categories"][$row["id"]]["children"][] = $child;
                }

                $children = array();
                if ($this->diafan->configmodules("childrens_elements"))
                {
                    $children = $this->diafan->get_children($row["id"], "shop_category");
                }
                $children[] = $row["id"];

                $result_img = DB::query_range(
                                "SELECT e.id, e.[name], e.cat_id, e.timeedit, e.[anons], e.site_id, e.no_buy, e.article, e.hit,"
                                . " e.new, e.action, e.is_file FROM {shop} AS e"
                                . " INNER JOIN {shop_category_rel} AS r ON e.id=r.element_id"
                                . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                                . " WHERE r.cat_id IN (%s) AND e.[act]='1' AND e.trash='0'"
                                . " AND (e.access='0'"
                                . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                . ")"
                                . " GROUP BY e.id ORDER BY e.sort ASC, e.id ASC", implode(',', $children), 0, $this->diafan->configmodules("count_list")
                );
                $this->result["categories"][$row["id"]]["rows"] = $this->get_elements($result_img);

                $this->result["categories"][$row["id"]]["link_all"] = $this->diafan->_route->link($row["site_id"], 'shop', $row["id"]);

                if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
                {
                    $this->result["categories"][$row["id"]]["img"] =
                            $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module, $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0, $this->result["categories"][$row["id"]]["link_all"], true);
                }
            }

            //сохранение кеша
            $this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
        }

        if ( ! empty($this->result["categories"]))
        {
            foreach ($this->result["categories"] as $cat => $rows)
            {
                if ( ! $rows)
                    continue;

                $this->result["categories"][$cat]["name"] = $this->diafan->_useradmin->get($rows["name"], 'name', $cat, 'shop_category', _LANG);
                $this->result["categories"][$cat]["anons"] = $this->diafan->_useradmin->get($rows["anons"], 'anons', $cat, 'shop_category', _LANG);
                foreach ($rows["children"] as $k => $row)
                {
                    $this->result["categories"][$cat]["children"][$k]["name"] =
                            $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'shop_category', _LANG);

                    $this->result["categories"][$cat]["children"][$k]["anons"] =
                            $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'shop_category', _LANG);
                }

                foreach ($rows["rows"] as $k => $row)
                {
                    $this->select_price($this->result["categories"][$cat]["rows"][$k]);

                    $this->useradmin($this->result["categories"][$cat]["rows"][$k]);

                    $this->result["categories"][$cat]["rows"][$k]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

                    $this->result["categories"][$cat]["rows"][$k]["show_rating"] =
                            $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
                }
            }
        }
		if($this->diafan->configmodules("theme_first_page"))
		{
			$this->result["theme"] = $this->diafan->configmodules("theme_first_page");
		}
		if($this->diafan->configmodules("view_first_page"))
		{
			$this->result["view"] = $this->diafan->configmodules("view_first_page");
		}
		else
		{
			$this->result["view"] = 'first_page';
		}
        return true;
    }

    /**
     * Генерирует данные для списка товаров в группе
     * 
     * @return boolean
     */
    public function list_category()
    {
        //кеширование
        $cache_meta = array(
            "name" => "list",
            "cat_id" => $this->diafan->cat,
            "language" => _LANG,
            "page" => $this->diafan->page > 1 ? $this->diafan->page : 1,
            "site_id" => $this->diafan->cid,
            "sort" => $this->diafan->sort,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => $this->get_tag_cache_discounts()
        );
        if ( ! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
        {
            $this->result["level"] = count($this->diafan->get_parents($this->diafan->cat, 'shop_category'));

            $this->result["path"] = $this->get_path('shop');

            $result = DB::query("SELECT id, [name], timeedit, [descr], [keywords], sort, parent_id, [title_meta], access, theme, view FROM {shop_category}"
                            . " WHERE [act]='1' and id=%d AND trash='0' AND site_id=%d"
                            . " ORDER BY sort ASC, id ASC", $this->diafan->cat, $this->diafan->cid);

            if ( ! $row = DB::fetch_array($result))
            {
                include_once ABSOLUTE_PATH . 'includes/404.php';
            }
			if (empty($row) || (!empty($row['access']) && !$this->access(0, $row['id'])))
			{
				include_once(ABSOLUTE_PATH.'includes/403.php');
			}

			if($row["theme"])
			{
				$this->result["theme"] = $row["theme"];
			}
			elseif($this->diafan->configmodules("theme_list"))
			{
				$this->result["theme"] = $this->diafan->configmodules("theme_list");
			}

			if($row["view"])
			{
				$this->result["view"] = $row["view"];
			}
			elseif($this->diafan->configmodules("view_list"))
			{
				$this->result["view"] = $this->diafan->configmodules("view_list");
			}
			else
			{
				$this->result["view"] = 'list';
			}

            $this->result["id"] = $row["id"];
            if ($this->diafan->configmodules("images_cat"))
            {
                $this->result["img"] = $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module, 0, 1, true);
            }

            if ($this->diafan->configmodules("childrens_elements"))
            {
                $children = $this->diafan->get_children($this->diafan->cat, "shop_category");
            }
            $children[] = $this->diafan->cat;

            ////navigation//
            $this->diafan->_paginator->page = $this->diafan->page;
            $this->diafan->_paginator->navlink = $this->diafan->_route->current_link("page");
            $this->diafan->_paginator->navlink_tpl = $this->diafan->_route->current_link("", array("page" => "%d"));
            $this->diafan->_paginator->nen = DB::query_result(
                            "SELECT COUNT(DISTINCT e.id) FROM {shop} AS e"
                            . " INNER JOIN {shop_category_rel} AS r ON e.id=r.element_id"
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                            . " WHERE e.[act]='1' AND e.trash='0'"
                            . " AND r.cat_id IN (%s)"
                            . " AND (e.access='0'"
                            . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                            . ")", implode(',', $children)
            );
            $links = $this->diafan->_paginator->get();
            ////navigation///

            $result_img = DB::query_range(
                            "SELECT s.id, s.[name], s.cat_id, s.timeedit, s.[anons], s.site_id, s.no_buy, s.article,"
                            . " s.hit, s.new, s.action, s.is_file FROM {shop} AS s"
							. ($this->diafan->sort == 1 || $this->diafan->sort == 2 ?
								" INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
								." AND (pr.date_start >=".time()." OR pr.date_start=0) AND pr.date_finish <=".time()
								." AND pr.currency_id=0"
								." AND pr.role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
								." AND pr.user_id".($this->diafan->_user->id ? " IN (0,".$this->diafan->_user->id.")" : "=0")
								.( !empty($this->order_summ) ? " AND pr.threshold <= ".$this->order_summ : '') 
								: '')
                            . ($this->sort_config['use_params_for_sort'] ? " LEFT JOIN {shop_param_element} AS sp  ON sp.element_id=s.id AND sp.trash='0' AND sp.param_id=" . $this->sort_config['param_ids'][$this->diafan->sort] : '')
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='shop'" : "")
                            . " INNER JOIN {shop_category_rel} AS r ON s.id=r.element_id AND r.cat_id IN (%s)"
                            . " WHERE s.[act]='1' AND s.trash='0'"
                            . " AND (s.access='0'"
                            . ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                            . ")"
                            . " GROUP BY s.id ORDER BY "
                            . ($this->diafan->sort ? $this->sort_config['sort_directions'][$this->diafan->sort] . ',' : '')
                            . "s.sort ASC, s.id ASC", implode(',', $children), $this->diafan->_paginator->polog, $this->diafan->_paginator->nastr
            );

            $this->result["rows"] = $this->get_elements($result_img);

            $this->result["paginator"] = $links;

            $this->result["titlemodule"] = $row["name"];
            $this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "shop_category");
            $this->result["title_meta"] = $row["title_meta"];
            $this->result["timeedit"] = $row["timeedit"];
            $this->result["keywords"] = $row["keywords"];
            $this->result["descr"] = $row["descr"];

            $previous = DB::fetch_array(DB::query(
                                    "SELECT c.[name], c.id FROM {shop_category} AS c"
                                    . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='shop'" : "")
                                    . " WHERE c.[act]='1' AND c.trash='0' AND c.site_id=%d"
                                    . " AND (c.sort<%d OR c.sort=%d AND c.id<%d) AND c.parent_id=%d"
                                    . " AND (c.access='0'"
                                    . ($this->diafan->_user->id ? " OR c.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                    . ")"
                                    . " ORDER BY c.sort DESC, c.id DESC LIMIT 1", $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]
                            ));
            if ($previous)
            {
                $this->result["previous"]["text"] = $previous["name"];
                $this->result["previous"]["id"] = $previous["id"];
                $this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $previous["id"]);
            }
            $next = DB::fetch_array(DB::query(
                                    "SELECT c.[name], c.id FROM {shop_category} AS c"
                                    . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='shop'" : "")
                                    . " WHERE c.[act]='1' AND c.trash='0' AND c.site_id=%d"
                                    . " AND (c.sort>%d OR c.sort=%d AND c.id>%d) AND c.parent_id=%d"
                                    . " AND (c.access='0'"
                                    . ($this->diafan->_user->id ? " OR c.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                    . ")"
                                    . " ORDER BY c.sort ASC, c.id ASC LIMIT 1", $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->cat, $row["parent_id"]
                            ));
            if ($next)
            {
                $this->result["next"]["text"] = $next["name"];
                $this->result["next"]["id"] = $next["id"];
                $this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $next["id"]);
            }

            $this->result["children"] = array();
            $result_children = DB::query(
                            "SELECT c.id, c.[name], c.[anons], c.site_id FROM {shop_category} AS c"
                            . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.cat_id=c.id AND a.module_name='shop'" : "")
                            . " WHERE c.[act]='1' AND c.parent_id=%d AND c.trash='0' AND c.site_id=%d"
                            . " AND (c.access='0'"
                            . ($this->diafan->_user->id ? " OR c.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                            . ")"
                            . " GROUP BY c.id ORDER BY c.sort ASC, c.id ASC", $row["id"], $this->diafan->cid
            );

            while ($child = DB::fetch_array($result_children))
            {
                $child["link"] = $this->diafan->_route->link($child["site_id"], 'shop', $child["id"]);
                if ($this->diafan->configmodules("images_cat") && $this->diafan->configmodules("list_img_cat"))
                {
                    $child["img"] = $this->diafan->_images->get($child["id"], $child["name"], $this->diafan->module, $this->diafan->configmodules("list_img_cat") == 1 ? 1 : 0, $child["link"], true);
                }
                unset($child["site_id"]);
                $this->result["children"][] = $child;
            }

            //сохранение кеша
            $this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
        }

        $text = DB::query_result("SELECT [text] FROM {shop_category} WHERE id=%d", $this->diafan->cat);
        $this->result["text"] = $this->diafan->_useradmin->get($text, 'text', $this->diafan->cat, 'shop_category', _LANG);

        if ($this->diafan->configmodules("cat") && $this->diafan->cat && $this->diafan->configmodules("comments_cat"))
        {
            $this->result["comments"] = $this->diafan->_comments->get();
        }

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
                $this->select_price($this->result["rows"][$i]);

                $this->useradmin($this->result["rows"][$i]);

                $this->result["rows"][$i]["tags"] = $this->diafan->configmodules("tags") ? $this->diafan->_tags->get($row["id"]) : '';

                $this->result["rows"][$i]["show_rating"] = $this->diafan->configmodules("rating") ? $this->diafan->_rating->get($row["id"]) : '';
            }
            if ( ! empty($this->result["previous"]["text"]))
            {
                $this->result["previous"]["text"] =
                        $this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'shop_category', _LANG);
            }
            if ( ! empty($this->result["next"]["text"]))
            {
                $this->result["next"]["text"] =
                        $this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'shop_category', _LANG);
            }
            foreach ($this->result["children"] as $id => $row)
            {
                $this->result["children"][$id]["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'shop_category', _LANG);
                $this->result["children"][$id]["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'shop_category', _LANG);
            }
            foreach ($this->result["path"] as $k => $path)
            {
                if ($k == 0)
                    continue;
                $this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'shop_category', _LANG);
            }
        }

        $this->result["link_sort"] = $this->get_sort_links($this->sort_config['sort_directions']);
        $this->result["sort_config"] = $this->sort_config;

        $this->result["paginator"] = $this->diafan->_tpl->get('get', 'paginator', $this->result["paginator"]);
		$this->result['shop_link']=$this->diafan->_route->module($this->diafan->module,true);

        return true;
    }

    /**
     * Генерирует данные для страницы товара
     * 
     * @return boolean
     */
    public function id()
    {
        //кеширование
        $cache_meta = array(
            "name" => "show",
            "cat_id" => $this->diafan->cat,
            "show" => $this->diafan->show,
            "language" => _LANG,
            "site_id" => $this->diafan->cid,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => $this->get_tag_cache_discounts()
        );
        if ( ! $this->result = $this->diafan->_cache->get($cache_meta, $this->diafan->module))
        {
            $row = DB::fetch_array(DB::query("SELECT id, [name], timeedit, cat_id, [keywords], [descr], sort, [title_meta],"
                                    . " no_buy, article, access, theme, view  FROM {shop}"
                                    . " WHERE [act]='1' AND id = %d AND trash='0' AND site_id=%d", $this->diafan->show, $this->diafan->cid));
            if (empty($row))
            {
                include ABSOLUTE_PATH . 'includes/404.php';
                return false;
            }

            if ( ! $this->diafan->rewrite_module && ($this->diafan->configmodules("cat") && $this->diafan->cat != $row["cat_id"]
                    || ! $this->diafan->configmodules("cat") && $this->diafan->cat))
            {
                include ABSOLUTE_PATH . 'includes/404.php';
                return false;
            }
            if ( ! empty($row['access']) && ! $this->access($row['id']))
            {
                include ABSOLUTE_PATH . 'includes/403.php';
                return false;
            }
            $this->diafan->cat = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;
            $row["cat_id"] = $this->diafan->configmodules("cat") ? $row["cat_id"] : 0;

            if ($this->diafan->configmodules("images"))
            {
                $this->result["img"] = $this->diafan->_images->get($row["id"], $row["name"], $this->diafan->module);
            }

            $this->get_price($row);
            foreach ($row as $id => $value)
            {
                $this->result[$id] = $value;
            }

			if(! $this->result["theme"] && $this->diafan->configmodules("theme_id"))
			{
				$this->result["theme"] = $this->diafan->configmodules("theme_id");
			}

			if(! $this->result["view"])
			{
				if($this->diafan->configmodules("view_id"))
				{
					$this->result["view"] = $this->diafan->configmodules("view_id");
				}
				else
				{
					$this->result["view"] = 'id';
				}
			}
            $this->result["currency"] = $this->diafan->configmodules("currency", "shop");
            $this->result["param"] = $this->get_param($row["id"], $this->diafan->cid);
            $this->result["titlemodule"] = $row["name"];
            $this->result["edit_titlemodule"] = array("id" => $row["id"], "table" => "shop");

            $previous = DB::fetch_array(DB::query(
                                    "SELECT e.[name], e.id, e.cat_id FROM {shop} AS e"
                                    . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                                    . " WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
                                    . ($this->diafan->configmodules("cat") ? " AND e.cat_id='" . $this->diafan->cat . "'" : '')
                                    . " AND (e.sort<%d OR e.sort=%d AND e.id<%d)"
                                    . " AND (e.access='0'"
                                    . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                    . ")"
                                    . " ORDER BY e.sort DESC, e.id DESC LIMIT 1", $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->show
                            ));
            if ($previous)
            {
                $this->result["previous"]["text"] = $previous["name"];
                if ( ! $this->diafan->configmodules("cat"))
                {
                    $previous["cat_id"] = 0;
                }
                $this->result["previous"]["id"] = $previous["id"];
                $this->result["previous"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $previous["cat_id"], $previous["id"]);
            }
            $next = DB::fetch_array(DB::query(
                                    "SELECT e.[name], e.id, e.cat_id FROM {shop} AS e"
                                    . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                                    . " WHERE e.[act]='1' AND e.trash='0' AND e.site_id=%d"
                                    . ($this->diafan->configmodules("cat") ? " AND e.cat_id='" . $this->diafan->cat . "'" : '')
                                    . " AND (e.sort>%d OR e.sort=%d AND e.id>%d)"
                                    . " AND (e.access='0'"
                                    . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                    . ")"
                                    . " ORDER BY e.sort ASC, e.id ASC LIMIT 1", $this->diafan->cid, $row["sort"], $row["sort"], $this->diafan->show
                            ));
            if ($next)
            {
                $this->result["next"]["text"] = $next["name"];
                if ( ! $this->diafan->configmodules("cat"))
                {
                    $next["cat_id"] = 0;
                }
                $this->result["next"]["id"] = $next["id"];
                $this->result["next"]["link"] = $this->diafan->_route->link($this->diafan->cid, "shop", $next["cat_id"], $next["id"]);
            }

            $this->result["path"] = $this->get_path('shop');

            //сохранение кеша
            $this->diafan->_cache->save($this->result, $cache_meta, $this->diafan->module);
        }
        $this->diafan->cat = $this->result["cat_id"];

        // увеличиваем количество просморов товара
		if(empty($_SESSION["shop_view"][$this->diafan->show]))
		{
			$_SESSION["shop_view"][$this->diafan->show] = true;
			DB::query("UPDATE {shop} SET counter_view=counter_view+1 WHERE id=%d", $this->diafan->show);
		}

        $row = DB::fetch_array(DB::query("SELECT [text], [anons], hit, new, action, is_file FROM {shop} WHERE id=%d", $this->diafan->show));

        $this->result["is_file"] = $this->diafan->configmodules("use_non_material_goods", "shop") ? $row["is_file"] : 0;
        $this->result["text"] = $row["text"];
        $this->result["anons"] = $row["anons"];
        $this->result["hit"] = $row["hit"];
        $this->result["new"] = $row["new"];
        $this->result["action"] = $row["action"];

        if ( ! empty($this->result["previous"]["text"]))
        {
            $this->result["previous"]["text"] =
                    $this->diafan->_useradmin->get($this->result["previous"]["text"], 'name', $this->result["previous"]["id"], 'shop', _LANG);
        }
        if ( ! empty($this->result["next"]["text"]))
        {
            $this->result["next"]["text"] =
                    $this->diafan->_useradmin->get($this->result["next"]["text"], 'name', $this->result["next"]["id"], 'shop', _LANG);
        }
        foreach ($this->result["path"] as $k => $path)
        {
            if ($k == 0)
                continue;
            $this->result["path"][$k]["name"] = $this->diafan->_useradmin->get($path["name"], 'name', $path["id"], 'shop_category', _LANG);
        }

        $this->select_price($this->result);
        $this->useradmin($this->result);

        if ($this->diafan->configmodules("rating"))
        {
            $this->result["show_rating"] = $this->diafan->_rating->get();
            $this->result["add_rating"] = $this->diafan->_rating->add();
        }

        if ($this->diafan->configmodules("comments"))
        {
            $this->result["comments"] = $this->diafan->_comments->get();
        }

        if ($this->diafan->configmodules("tags"))
        {
            $this->result["tags"] = $this->diafan->_tags->get();
        }

        return true;
    }

    /**
     * Генерирует данные для шаблонной функции: блок товаров
     * 
     * @param integer $count количество файлов
     * @param integer $cat_id категория
     * @param integer $site_id страница сайта
     * @param string $sort сортировка date - по дате, rand - случайно, price - по цене, sale - по количеству продаж
     * @param integer $images количество фотографий
     * @param string $param дополнительные параметры
     * @return boolean
     */
    public function show_block($count, $cat_id, $site_id, $sort, $images, $param, $hits_only, $action_only, $new_only)
    {
        //кеширование
        $cache_meta = array(
            "name" => "block",
            "cat_id" => $cat_id,
            "count" => $count,
            "language" => _LANG,
            "site_id" => $site_id,
            "images" => $images,
            "param" => $param,
            "sort" => $sort,
            "hits_only" => (int) $hits_only,
            "action_only" => (int) $action_only,
            "new_only" => (int) $new_only,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => ($sort == "rand" || $sort == "sale" ? "" : $this->get_tag_cache_discounts())
        );

        if ($sort == "rand" || $sort == "sale" || ! $this->result = $this->diafan->_cache->get($cache_meta, "shop"))
        {

            if ( ! $site_id)
            {
                $site_id = DB::query_result(
                                "SELECT s.id FROM {site} AS s"
                                . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
                                . " WHERE s.addmodule='shop' AND s.trash='0' AND s.[act]='1' AND s.block='0'"
                                . " AND (s.access='0'"
                                . ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                . ") LIMIT 1"
                );
            }
            else
            {
                if (DB::query_result("SELECT access FROM {site} WHERE id=%d", $site_id))
                {
                    if ( ! $this->access($site_id, 0, "site"))
                    {
                        return FALSE;
                    }
                }
            }
            if ( ! $site_id)
            {
                return FALSE;
            }
            if ($cat_id && $this->diafan->configmodules("cat", "shop"))
            {
                if (DB::query_result("SELECT access FROM {shop_category} WHERE id=%d", $cat_id))
                {
                    if ( ! $this->access(0, $cat_id, "shop"))
                    {
                        return FALSE;
                    }
                }
                if ($this->diafan->configmodules("childrens_elements", "shop"))
                {
                    $children = $this->diafan->get_children($cat_id, "shop_category");
                }
                $children[] = $cat_id;
            }
            $params = array();
            if ($param)
            {
                $param = explode('&', $param);
                foreach ($param as $p)
                {
                    list($id, $value) = explode('=', $p, 2);
                    $id = preg_replace('/[^0-9]+/', '', $id);
                    if ( ! empty($params[$id]))
                    {
                        if (is_array($params[$id]))
                        {
                            $params[$id][] = $value;
                        }
                        else
                        {
                            $params[$id] = array($params[$id], $value);
                        }
                    }
                    else
                    {
                        $params[$id] = $value;
                    }
                }
            }
            $where = '';
            $values = array();
            foreach ($params as $id => $value)
            {
                if (is_array($value))
                {
                    $where .= "
					INNER JOIN {shop_param_element} AS pe" . $id . " ON pe" . $id . ".element_id=e.id AND pe" . $id . ".param_id='" . $id . "'"
                            . " AND pe" . $id . ".trash='0' AND (";
                    foreach ($value as $val)
                    {
                        if ($value[0] != $val)
                        {
                            $where .= " OR ";
                        }
                        $where .= "pe" . $id . ".value='%h'";
                        $values[] = $val;
                    }
                    $where .= ")";
                }
                else
                {
                    $where .= "
					INNER JOIN {shop_param_element} AS pe" . $id . " ON pe" . $id . ".element_id=e.id AND pe" . $id . ".param_id='" . $id . "'"
                            . " AND pe" . $id . ".trash='0' AND pe" . $id . ".value='%h'";
                    $values[] = $value;
                }
            }

            if ($sort == "rand")
            {
                $max_count = DB::query_result("SELECT COUNT(DISTINCT e.id) FROM {shop} as e"
                                . ($cat_id && $this->diafan->configmodules("cat", "shop") ? " INNER JOIN {shop_category_rel} as c ON c.element_id=e.id"
                                        . " AND c.cat_id IN (" . implode(',', $children) . ")" : ''
                                )
                                . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                                . $where
                                . " WHERE e.[act]='1' AND e.trash='0'"
                                . " AND (e.access='0'"
                                . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                . ")"
                                . ($site_id ? " AND e.site_id=" . $site_id : ''), $values
                );
                $rands = array();
                for ($i = 1; $i <= min($max_count, $count); $i ++ )
                {
                    do
                    {
                        $rand = mt_rand(0, $max_count - 1);
                    }
                    while (in_array($rand, $rands));
                    $rands[] = $rand;
                }
            }
            else
            {
                $rands[0] = 1;
            }
            $this->result["rows"] = array();
			
			switch($sort)
			{
				case 'price':
					$order = ' ORDER BY MIN(pr.price) ASC';
					break;

				case 'sale':
					$order = ' ORDER BY count_sale DESC';
					break;

				case 'date':
					$order = ' ORDER BY e.id DESC';
					break;

				default :
					$order  = '';
			}

            foreach ($rands as $rand)
            {
                $result = DB::query("SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id, e.no_buy, e.article,
                                e.hit, e.new, e.action, e.is_file".($sort == "sale" ? ", COUNT(g.id) AS count_sale" : "")."
								FROM {shop} AS e"
								. ($sort == "sale" ? " INNER JOIN {shop_order_goods} AS g ON g.good_id=e.id AND g.trash='0'" : '')
								. ($sort == "price" ? " INNER JOIN {shop_price} AS pr ON pr.good_id=e.id AND pr.trash='0'"
								." AND (pr.date_start >=".time()." OR pr.date_start=0) AND pr.date_finish <=".time()
								." AND pr.currency_id=0"
								." AND pr.role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
								." AND pr.user_id".($this->diafan->_user->id ? " IN (0,".$this->diafan->_user->id.")" : "=0")
								." AND pr.threshold <= ".$this->order_summ : '')
                                . ($cat_id && $this->diafan->configmodules("cat", "shop") ? " INNER JOIN {shop_category_rel} as c ON c.element_id=e.id"
                                        . " AND c.cat_id IN (" . implode(',', $children) . ")" : ''
                                )
                                . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                                . $where
                                . " WHERE e.[act]='1' AND e.trash='0'"
                                . ($hits_only ? " AND hit='1' " : "")
                                . ($action_only ? " AND action='1' " : "")
                                . ($new_only ? " AND new='1' " : "")
                                . ($site_id ? " AND e.site_id=" . $site_id : '')
                                . " AND (e.access='0'"
                                . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                                . ")"
                                . " GROUP BY e.id"
                                . $order
                                . ' LIMIT '
                                . ($sort == "rand" ? $rand : 0) . ', '
                                . ($sort == "rand" ? 1 : $count), $values);
                $this->result["rows"] = array_merge($this->result["rows"], $this->get_elements($result, $images, "block"));
            }

            if ( ! empty($this->result["rows"]) && $cat_id && $this->diafan->configmodules("cat", "shop"))
            {
                $cat = DB::fetch_array(DB::query("SELECT [name], site_id, id FROM {shop_category} WHERE id=%d", $cat_id));

                $this->result["name"] = $cat["name"];
                $this->result["link_all"] = $this->diafan->_route->link($cat["site_id"], 'shop', $cat["id"]);
                $this->result["category"] = true;
            }
            elseif ( ! empty($this->result["rows"]) && $site_id)
            {
                $this->result["name"] = DB::query_result("SELECT [name] FROM {site} WHERE id=%d", $site_id);
                $this->result["link_all"] = $this->diafan->_route->link($site_id);
                $this->result["category"] = false;
            }

            //сохранение кеша
            if ($sort != "rand")
            {
                $this->diafan->_cache->save($this->result, $cache_meta, "shop");
            }
        }

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
                $this->select_price($this->result["rows"][$i]);
                $this->useradmin($this->result["rows"][$i]);
            }
        }

        return true;
    }

    /**
     * Генерирует данные для шаблонной функции: блок связанных товаров
     * 
     * @param integer $count количество файлов
     * @param integer $images количество фотографий
     * @return boolean
     */
    public function show_block_rel($count, $images)
    {
        //кеширование
        $cache_meta = array(
            "name" => "block_rel",
            "count" => $count,
            "language" => _LANG,
            "good_id" => $this->diafan->show,
            "images" => $images,
            "role_id" => $this->diafan->_user->id ? $this->diafan->_user->role_id : 0,
			"discounts" => $this->get_tag_cache_discounts()
        );

        if (! $this->result = $this->diafan->_cache->get($cache_meta, "shop"))
        {
            $this->result["rows"] = array();

			$result = DB::query_range(
					"SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id, e.no_buy, e.article,"
					." e.hit, e.new, e.action, e.is_file FROM {shop} AS e"
					. " INNER JOIN {shop_rel} AS r ON e.id=r.good_id AND r.rel_good_id=%d OR e.id=r.rel_good_id AND r.good_id=%d"
					. ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
					. " WHERE e.[act]='1' AND e.trash='0'"
					. " AND (e.access='0'"
					. ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
					. ")"
					. " GROUP BY e.id"
					. ' ORDER BY e.id DESC',
					$this->diafan->show, $this->diafan->show, 0, $count
				);
			$this->result["rows"] = $this->get_elements($result, $images, "block");
			$this->diafan->_cache->save($this->result, $cache_meta, "shop");
		}

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
                $this->select_price($this->result["rows"][$i]);
                $this->useradmin($this->result["rows"][$i]);
            }
        }

        return true;
    }

    /**
     * Генерирует данные для шаблонной функции: блок товаров, которые обычно покупают с текущим
     * 
     * @param integer $count количество файлов
     * @param integer $images количество фотографий
     * @return boolean
     */
    public function show_block_order_rel($count, $images)
    {
		$this->result["rows"] = array();

		$result = DB::query_range(
				"SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id, e.no_buy, e.article,"
				." e.hit, e.new, e.action, e.is_file, COUNT(g.good_id) AS count_good FROM {shop} AS e"
				. " INNER JOIN {shop_order_goods} AS g ON g.good_id=e.id AND g.good_id<>%d"
				. " INNER JOIN {shop_order_goods} AS eg ON eg.order_id=g.order_id AND eg.good_id=%d"
				. ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
				. " WHERE e.[act]='1' AND e.trash='0'"
				. " AND (e.access='0'"
				. ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
				. ")"
				. " GROUP BY e.id"
				. ' ORDER BY count_good DESC',
				$this->diafan->show, $this->diafan->show, 0, $count
			);
		$this->result["rows"] = $this->get_elements($result, $images, "block");

        if ($this->result["rows"])
        {
            foreach ($this->result["rows"] as $i => $row)
            {
				$this->select_price($this->result["rows"][$i]);
                $this->useradmin($this->result["rows"][$i]);
            }
        }

        return true;
    }

    /**
     * Генерирует данные для вывода тегов
     * 
     * @param integer $element_id  номер товара
     * @return array
     */
    public function tags($element_id)
    {
        $result = DB::query("SELECT e.id, e.[name], e.[anons], e.cat_id, e.timeedit, e.site_id, e.no_buy, e.article FROM {shop} AS e"
                        . ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=e.id AND a.module_name='shop'" : "")
                        . " WHERE e.[act]='1' AND e.trash='0' AND e.id=%d"
                        . " AND (e.access='0'"
                        . ($this->diafan->_user->id ? " OR e.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
                        . ")", $element_id);
        $this->result["rows"] = $this->get_elements($result, "list", "block");

        if ( ! $this->result["rows"])
        {
            return false;
        }
        for ($i = 0; $i < count($this->result["rows"]); $i ++ )
        {
            $this->select_price($this->result["rows"][$i]);
        }

        return $this->get_result();
    }

    /**
     * Генерирует контент для шаблонной функции: форма поиска по товарам
     *
     * @param integer|string $cat_id номер категории, в которой осуществляется поиск
     * @param integer $site_id страница сайта, к которой прикреплен модуль Магазин
     * @return array
     */
    public function show_search($cat_id, $site_id)
    {
        if ( ! $site_id)
        {
            if ($this->diafan->module == "shop")
            {
                $site_id = $this->diafan->cid;
            }
            else
            {
                $site_id = DB::query_result(
					"SELECT s.id FROM {site} AS s"
					. ($this->diafan->_user->id ? " LEFT JOIN {access} AS a ON a.element_id=s.id AND a.module_name='site'" : "")
					. " WHERE s.addmodule='shop' AND s.trash='0' AND s.[act]='1' AND s.block='0'"
					. " AND (s.access='0'"
					. ($this->diafan->_user->id ? " OR s.access='1' AND a.role_id=" . $this->diafan->_user->role_id : '')
					. ") LIMIT 1"
                );
            }
        }
        if ( ! $site_id)
        {
            return FALSE;
        }

        if ($cat_id === 'current')
        {
            if ($this->diafan->cat && $site_id == $this->diafan->cid)
            {
                $cat_id = $this->diafan->cat;
            }
            elseif ( ! empty($_GET["catid"]) && $site_id == $this->diafan->cid)
            {
                $this->diafan->cat = (int) $this->diafan->get_param($_GET, "catid", '', 2);
                $cat_id = $this->diafan->cat;
            }
            else
            {
                $cat_id = 0;
            }
        }
        if ( ! $this->diafan->configmodules("cat", "shop", $site_id))
        {
            $cat_id = 0;
        }
        $this->result["path"] = $this->diafan->_route->link($site_id);
        $this->result["cat_id"] = $cat_id;

        if ($this->diafan->configmodules("search_name", "shop", $site_id))
        {
            $this->result["name"] = array(
                "name" => 1,
                "value" => ! empty($_GET["n"]) ? trim(htmlspecialchars(stripslashes($_GET["n"]))) : ''
            );
        }

        if ($this->diafan->configmodules("search_article", "shop", $site_id))
        {
            $this->result["article"] = array(
                "article" => 1,
                "value" => ! empty($_GET["a"]) ? trim(htmlspecialchars(stripslashes($_GET["a"]))) : ''
            );
        }

        if ($this->diafan->configmodules("search_text", "shop", $site_id))
        {
            $this->result["text"] = array(
                "name" => 1,
                "value" => ! empty($_GET["d"]) ? trim(htmlspecialchars(stripslashes($_GET["d"]))) : ''
            );
        }

        if ($this->diafan->configmodules("search_price", "shop", $site_id))
        {
            $this->result["price"] = array(
                "name" => 1,
                "value1" => (int) $this->diafan->get_param($_GET, "pr1", '', 2),
                "value2" => (int) $this->diafan->get_param($_GET, "pr2", '', 2)
            );
        }

        $result = DB::query("SELECT DISTINCT(p.id), p.type, p.[name] FROM {shop_param} as p "
                        . " INNER JOIN {shop_param_category_rel} AS c ON p.id=c.element_id "
                        . ($this->diafan->configmodules("cat", "shop", $site_id) ? " AND (c.cat_id=%d OR c.cat_id=0)" : "")
                        . " WHERE p.search='1' AND p.trash='0' ORDER BY p.sort ASC", $cat_id);

        while ($row = DB::fetch_array($result))
        {
            if ($row["type"] == 'text' || $row["type"] == 'textarea' || $row["type"] == 'editor')
            {
                $row["value"] = ! empty($_GET["p" . $row["id"]]) ? trim(htmlspecialchars(stripslashes($_GET["p" . $row["id"]]))) : '';
            }
            elseif ($row["type"] == 'date' || $row["type"] == 'datetime')
            {
                $row["value1"] = $this->diafan->get_param($_GET, "p" . $row["id"] . "_1", '', 1);
                $row["value2"] = $this->diafan->get_param($_GET, "p" . $row["id"] . "_2", '', 1);
            }
            elseif ($row["type"] == 'numtext')
            {
                $row["value1"] = (int) $this->diafan->get_param($_GET, "p" . $row["id"] . "_1", '', 2);
                $row["value2"] = (int) $this->diafan->get_param($_GET, "p" . $row["id"] . "_2", '', 2);
            }
            elseif ($row["type"] == 'checkbox')
            {
                $row["value"] = (int) $this->diafan->get_param($_GET, "p" . $row["id"], '', 2);
            }
            elseif ($row["type"] == 'select' || $row["type"] == 'multiple')
            {
                $result_select = DB::query("SELECT [name], id FROM {shop_param_select} WHERE param_id=%d ORDER BY sort ASC", $row["id"]);
                while ($row_select = DB::fetch_array($result_select))
                {
                    $row["select_array"][$row_select["id"]] = $row_select["name"];
                }
                if ( ! empty($_GET["p" . $row["id"]]) && ! is_array($_GET["p" . $row["id"]]))
                {
                    $row["value"][] = (int) $this->diafan->get_param($_GET, "p" . $row["id"], 0, 2);
                }
                elseif ( ! empty($_GET["p" . $row["id"]]) && is_array($_GET["p" . $row["id"]]))
                {
                    foreach ($_GET["p" . $row["id"]] as $val)
                    {
                        $row["value"][] = intval($val);
                    }
                }
                else
                {
                    $row["value"] = array();
                }
            }
            unset($row["param"]);
            $this->result["rows"][] = $row;
        }
        return $this->result;
    }

    /**
     * Шаблонная функция: форма активации купона
	 * 
     * @return boolean
     */
    public function show_add_coupon()
    {
		$result["error"] = $this->get_error("shop");
		$result["count"] = DB::query_result("SELECT COUNT(*) FROM {shop_discount} WHERE coupon<>'' AND user_id=%d AND trash='0' AND act='1'", $this->diafan->_user->id);
        return $result;
    }

    /**
     * Форматирует данные о товаре для списка товаров
     *
     * @param resource $result результат выполнения SQL-запроса
     * @param string $images показывать фотографии
     * @param string $function функция, для которой генерируется список товаров
     * @return array
     */
    public function get_elements($result, $images = 'list', $function = 'list')
    {
        if (empty($this->result["timeedit"]))
        {
            $this->result["timeedit"] = '';
        }
        $rows = array();
        while ($row = DB::fetch_array($result))
        {
            if ( ! $this->diafan->configmodules("cat", "shop", $row["site_id"]))
            {
                $row["cat_id"] = 0;
            }
            if ($row["timeedit"] < $this->result["timeedit"])
            {
                $this->result["timeedit"] = $row["timeedit"];
            }
            unset($row["timeedit"]);

            $row["link"] = $this->diafan->_route->link($row["site_id"], "shop", $row["cat_id"], $row["id"]);

            if ($this->diafan->configmodules("images", "shop", $row["site_id"]))
            {
                if ($images === 'list' && $this->diafan->configmodules("list_img", "shop", $row["site_id"]))
                {
                    $row["img"] = $this->diafan->_images->get($row["id"], $row["name"], 'shop', $this->diafan->configmodules("list_img", "shop", $row["site_id"]) == 1 ? 1 : 0, $row["link"]);
                }
                elseif ($images != 'list' && $images)
                {
                    $row["img"] = $this->diafan->_images->get($row["id"], $row["name"], 'shop', $images, $row["link"]);
                }
            }
            $this->get_price($row);

            $row["param"] = $this->get_param($row["id"], $row["site_id"], $function);
            $row["is_file"] = $this->diafan->configmodules("use_non_material_goods", "shop") ? $row["is_file"] : 0;

            $rows[] = $row;
        }
		if(! isset($this->result["currency"]))
		{
			$this->result["currency"] = $this->diafan->configmodules("currency", "shop");
		}
        return $rows;
    }

    /**
     * Формирует данные о цене товара
     *
     * @param array $row данные о товаре
     * @return boolean true
     */
    private function get_price(&$row)
    {
		$price = array();
		$count = 0;
		$rows = $this->diafan->_shop->price_get_all($row["id"], $this->diafan->_user->id);
		foreach($rows as $row_price)
		{
			$row_price["param"] = array();
			$result_param = DB::query("SELECT param_id, param_value FROM {shop_price_param} WHERE price_id=%d", $row_price["price_id"]);
			while ($row_param = DB::fetch_array($result_param))
			{
				$row["param_multiple"][$row_param["param_id"]][$row_param["param_value"]] = TRUE;
				$row_price["param"][] = array("id" => $row_param["param_id"], "value" => $row_param["param_value"]);
			}
			$count += $row_price["count_goods"];
			$row_price["count"] = $this->diafan->configmodules("use_count_goods", "shop") ? $row_price["count_goods"] : true;
			$price[] = $row_price;
		}
		// если цена одна, параметры не заданы
		if(count($price) == 1)
		{
			$row["price_id"] = $price[0]["price_id"];
			$row["price_param"] = $price[0]["param"];
			$row["price"] = $price[0]["price"];
			if($price[0]["price"] != $price[0]["old_price"])
			{
				$row["count"] = $this->diafan->configmodules("use_count_goods", "shop") ? $price[0]["count_goods"] : true;
				$row["old_price"] = $price[0]["old_price"];
			}
		}
		else
		{
			$row["price_arr"] = $price;
			$row["count"] = $this->diafan->configmodules("use_count_goods", "shop") ? $count : true;
		}
		$row["discount"] = !empty($price[0]) ? $price[0]["discount"] : 0;
		$row["discount_finish"] = !empty($price[0]["date_finish"]) ? $this->format_date($price[0]["date_finish"], "shop") : '';

		// массив всех характеристик, доступных к выбору при заказе
		if (! isset($this->result["depends_param"]))
		{
			$this->result["depends_param"] = array();
			$result = DB::query("SELECT id, [name] FROM {shop_param} WHERE `type`='multiple' AND required='1' AND trash='0' ORDER BY sort ASC");
			while ($row_param = DB::fetch_array($result))
			{
				$row_param["values"] = array();
				$result_value = DB::query("SELECT id, [name] FROM {shop_param_select} WHERE param_id=%d AND trash='0'", $row_param["id"]);
				while ($row_value = DB::fetch_array($result_value))
				{
					$row_param["values"][] = $row_value;
				}
				$this->result["depends_param"][] = $row_param;
			}
		}
        return true;
    }

    /**
     * Формирует метку кэша на основе данные о скидках
     * 
     * @return string
     */
    private function get_tag_cache_discounts()
    {
        $discounts = array();
		$min = round(date("i")/5)*5;
		$time = mktime(date("H"), $min, 0);
        $result = DB::query("SELECT id FROM {shop_discount} WHERE trash='0' AND act='1'"
				." AND threshold < %d"
				." AND role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
				." AND ".($this->diafan->_user->id ? "  (user_id=".$this->diafan->_user->id." OR user_id=0 AND coupon='')" : "user_id=0 AND coupon=''")
				." AND (date_start >=%d OR date_start=0) AND date_finish <= %d"
				." ORDER BY id ASC", $this->order_summ, $time, $time);
        while ($dis = DB::fetch_array($result))
        {
            $discounts[] = $dis["id"];
        }
		return implode(",", $discounts);
	}

    /**
     * Возвращает результаты, сформированные в моделе
     * 
     * @return void
     */
    public function get_result()
    {
		$this->result["cart_link"] = $this->diafan->_route->module("cart");
		$this->result["wishlist_link"] = $this->diafan->_route->module("wishlist");
        $this->result["buy"] =  ! $this->diafan->configmodules('not_buy', "shop") && (! $this->diafan->configmodules('security_user', "shop") || $this->diafan->_user->id) ? true : false;

		return $this->result;
    }

    /**
     * Задает выделение параметров, учитываемый при покупке товара
     *
     * @param array $row данные о товаре
     * @return boolean true
     */
    private function select_price(&$row)
    {
		$row["count_in_cart"] = 0;
        if ( ! empty($row["price_arr"]))
        {
            $new_params = array();
            foreach ($row["price_arr"] as $id => $price)
            {
                $row["price_arr"][$id]["count_in_cart"] = 0;
				$keys = array();
				foreach($price["param"] as $param)
				{
					$keys[$param["id"]] = $param["value"];
				}
				asort($keys);
				$row_key = '|';
				foreach ($keys as $key => $value)
				{
					$row_key .= $key.'='.$value.'|';
				}
                if ( ! empty($_SESSION["cart"][$row["id"]]))
                {
                    foreach ($_SESSION["cart"][$row["id"]] as $key => $cart)
                    {
                        if ($key == $row_key)
                        {
                            $row["price_arr"][$id]["count_in_cart"] += $cart["count"];
                            $row["count_in_cart"] += $cart["count"];
                        }
                    }
                }
				$row["price_arr"][$id]["price"] = $this->diafan->format_price($row["price_arr"][$id]["price"]);
				if(! empty($row["price_arr"][$id]["old_price"]))
				{
					$row["price_arr"][$id]["old_price"] = $this->diafan->format_price($row["price_arr"][$id]["old_price"]);
				}
            }
        }
        else
        {
            if ( ! empty($_SESSION["cart"][$row["id"]]))
            {
                foreach ($_SESSION["cart"][$row["id"]] as $cart)
                {
                    $row["count_in_cart"] += $cart["count"];
                }
            }
			if(! empty($row["price"]))
			{
				$row["price"] = $this->diafan->format_price($row["price"]);
			}
			if(! empty($row["old_price"]))
			{
				$row["old_price"] = $this->diafan->format_price($row["old_price"]);
			}
        }

        return true;
    }

    /**
     * Получает дополнительные характеристики товара
     * 
     * @param integer $id номер товара
     * @param integer $site_id номер страницы, к которой прикреплен товар
     * @param string $function функция, для которой выбираются параметры
     * @return array
     */
    private function get_param($id, $site_id, $function = "id")
    {
        global $param_select, $param_select_page;
        $values = array();
        $rvalues = array();
        $result_el = DB::query("SELECT e.value as rvalue, e.[value], e.param_id, e.id FROM {shop_param_element} as e"
                        . " LEFT JOIN {shop_param_select} as s ON e.param_id=s.param_id AND e.param_id=s.id"
                        . " WHERE e.element_id=%d GROUP BY e.id ORDER BY s.sort ASC", $id);
        while ($row_el = DB::fetch_array($result_el))
        {
            $values[$row_el["param_id"]][] = $row_el;
        }
        $result = DB::query("SELECT p.id, p.[name], p.type, p.page, p.measure_unit FROM {shop_param} as p "
                        . ($this->diafan->configmodules("cat", "shop", $site_id) ? " INNER JOIN {shop_category_rel} as c ON c.element_id=" . $id : "")
                        . " INNER JOIN {shop_param_category_rel} as cp ON cp.element_id=p.id "
                        . ($this->diafan->configmodules("cat", "shop", $site_id) ?
                                " AND (cp.cat_id=c.cat_id OR cp.cat_id=0) " : "")
                        . " WHERE p.trash='0' "
                        . ($function == "block" ? " AND p.block='1'" : '')
                        . ($function == "list" ? " AND p.list='1'" : '')
                        . " GROUP BY p.id ORDER BY p.sort ASC"
        );

        $param = array();
        while ($row = DB::fetch_array($result))
        {
            switch ($row["type"])
            {
                case "text":
                case "textarea":
                case "editor":
                    if ( ! empty($values[$row["id"]][0]["value"]))
                    {
                        $param[] = array(
                            "id" => $row["id"],
                            "name" => $row["name"],
                            "value" => $values[$row["id"]][0]["value"],
                            "value_id" => $values[$row["id"]][0]["id"],
                            "type" => $row["type"]
                        );
                    }
                    break;
                case "date":
                    if ( ! empty($values[$row["id"]][0]["rvalue"]))
                    {
                        $param[] = array(
                            "id" => $row["id"],
                            "name" => $row["name"],
                            "value" => $this->diafan->formate_from_date($values[$row["id"]][0]["rvalue"]),
                            "value_id" => $values[$row["id"]][0]["id"],
                            "type" => $row["type"]
                        );
                    }
                    break;
                case "datetime":
                    if ( ! empty($values[$row["id"]][0]["rvalue"]))
                    {
                        $param[] = array(
                            "id" => $row["id"],
                            "name" => $row["name"],
                            "value" => $this->diafan->formate_from_datetime($values[$row["id"]][0]["rvalue"]),
                            "value_id" => $values[$row["id"]][0]["id"],
                            "type" => $row["type"]
                        );
                    }
                    break;
                case "select":
                    $value = ! empty($values[$row["id"]][0]["rvalue"]) ? $values[$row["id"]][0]["rvalue"] : '';
                    if ($value)
                    {
                        if (empty($param_select[$row["id"]][$value]))
                        {
                            $param_select[$row["id"]][$value] = DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d AND param_id=%d", $values[$row["id"]][0]["rvalue"], $row["id"]);
                        }
                        if ($row["page"])
                        {
                            if (empty($param_select_page[$row["id"]][$value]))
                            {
                                $param_select_page[$row["id"]][$value] = $this->diafan->_route->link($site_id, "shop", 0, 0, $value);
                            }
                            $link = $param_select_page[$row["id"]][$value];
                            $param[] = array("id" => $row["id"], "name" => $row["name"], "type" => $row["type"], "value" => $param_select[$row["id"]][$value], "link" => $link);
                        }
                        else
                        {
                            $param[] = array("id" => $row["id"], "name" => $row["name"], "type" => $row["type"], "value" => $param_select[$row["id"]][$value]);
                        }
                    }
                    break;
                case "multiple":
                    if ( ! empty($values[$row["id"]]))
                    {
                        $value = array();
                        foreach ($values[$row["id"]] as $val)
                        {
                            if (empty($param_select[$row["id"]][$val["rvalue"]]))
                            {
                                $param_select[$row["id"]][$val["rvalue"]] =
                                        DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d AND param_id=%d", $val["rvalue"], $row["id"]);
                            }
                            if ($row["page"])
                            {
                                if ($this->diafan->module == 'shop' && $this->diafan->param == $val["rvalue"])
                                {
                                    $link = '';
                                }
                                else
                                {
                                    if (empty($param_select_page[$row["id"]][$val["rvalue"]]))
                                    {
                                        $param_select_page[$row["id"]][$val["rvalue"]] = $this->diafan->_route->link($site_id, "shop", 0, 0, $val["rvalue"]);
                                    }
                                    $link = $param_select_page[$row["id"]][$val["rvalue"]];
                                }
                                $value[] = array("id" => $row["id"], "name" => $param_select[$row["id"]][$val["rvalue"]], "link" => $link);
                            }
                            else
                            {
                                $value[] = $param_select[$row["id"]][$val["rvalue"]];
                            }
                        }
                        $param[] = array("id" => $row["id"], "name" => $row["name"], "type" => $row["type"], "value" => $value);
                    }
                    break;
                case "checkbox":
                    $value = ! empty($values[$row["id"]][0]["rvalue"]) ? 1 : 0;
                    if ( ! isset($param_select[$row["id"]][$value]))
                    {
                        $param_select[$row["id"]][$value] =
                                DB::query_result("SELECT [name] FROM {shop_param_select} WHERE value=%d AND param_id=%d", $value, $row["id"]);
                    }
                    if ( ! $param_select[$row["id"]][$value] && $value == 1)
                    {
                        $param[] = array("id" => $row["id"], "name" => $row["name"], "type" => $row["type"], "value" => '');
                    }
                    else
                    {
                        $param[] = array("id" => $row["id"], "name" => $row["name"], "type" => $row["type"], "value" => $param_select[$row["id"]][$value]);
                    }
                    break;
                case "title":
                    $param[] = array(
                        "id" => $row["id"],
                        "name" => $row["name"],
                        "type" => $row["type"],
                        "value" => ''
                    );
                    break;
                case "numtext":
                    if ( ! empty($values[$row["id"]][0]["rvalue"]))
                    {
                        $param[] = array(
                            "id" => $row["id"],
                            "name" => $row["name"],
                            "value" => $values[$row["id"]][0]["rvalue"],
                            "value_id" => $values[$row["id"]][0]["id"],
                            "type" => $row["type"],
                            "measure_unit" => $row["measure_unit"]
                        );
                    }
                    break;
                default:
                    if ( ! empty($values[$row["id"]][0]["rvalue"]))
                    {
                        $param[] = array(
                            "id" => $row["id"],
                            "name" => $row["name"],
                            "value" => $values[$row["id"]][0]["rvalue"],
                            "value_id" => $values[$row["id"]][0]["id"],
                            "type" => $row["type"]
                        );
                    }
                    break;
            }
        }
        return $param;
    }

    /**
     * Формирует SQL-запрос при поиске по товарам
     * 
     * @return boolean true
     */
    private function get_where(&$where, &$where_param, &$values, &$getnav, &$group)
    {
        $where = ' AND s.site_id=%d';
		$values[] = $this->diafan->cid;
		$values_param = array();
        if (isset($_GET["catid"]))
        {
            $this->diafan->cat = (int) $this->diafan->get_param($_GET, "catid", null, 2);
            $catarr = array(0);
            $getnav = '?action=search&catid=' . $this->diafan->cat;
            if ($this->diafan->cat)
            {
                if ($this->diafan->configmodules("childrens_elements"))
                {
                    $children = $this->diafan->get_children($this->diafan->cat, "shop_category");
                }
                $where_param .= " INNER JOIN {shop_category_rel} AS c ON s.id=c.element_id AND "
                        . ($this->diafan->configmodules("childrens_elements") ? 'c.cat_id in (' . implode(',', $children) . ')' : 'c.cat_id=' . $this->diafan->cat);
            }

            if ( ! empty($_GET["n"]) && $this->diafan->configmodules("search_name"))
            {
                $where .= " AND s.name" . _LANG . " LIKE '%%%h%%'";
                $_GET["n"] = trim($this->diafan->get_param($_GET, "n", '', 1));
                $values[] = $_GET["n"];
                $getnav .= '&n=' . $_GET["n"];
            }

            if ( ! empty($_GET["a"]) && $this->diafan->configmodules("search_article"))
            {
                $where .= " AND s.article" . _LANG . "='%h'";
                $_GET["a"] = trim($this->diafan->get_param($_GET, "a", '', 1));
                $values[] = $_GET["a"];
                $getnav .= '&a=' . $_GET["a"];
            }

            if ( ! empty($_GET["d"]) && $this->diafan->configmodules("search_text"))
            {
                $where .= " AND s.text" . _LANG . " LIKE '%%%h%%'";
                $_GET["d"] = trim($this->diafan->get_param($_GET, "d", '', 1));
                $values[] = $_GET["d"];
                $getnav .= '&d=' . $_GET["d"];
            }

            if ( ! empty($_GET["pr1"]) || ! empty($_GET["pr2"]))
            {
                if ( ! empty($_GET["pr1"]))
                {
                    $pr1 = (int) $this->diafan->get_param($_GET, "pr1", '', 2);
                    $getnav .= '&pr1=' . $pr1;
                }
                if ( ! empty($_GET["pr2"]))
                {
                    $pr2 = (int) $this->diafan->get_param($_GET, "pr2", '', 2);
                    $getnav .= '&pr2=' . $pr2;
                }
                $where_param .= " INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
					." AND (pr.date_start >=".time()." OR pr.date_start=0) AND pr.date_finish <=".time()
					." AND pr.currency_id=0"
					." AND pr.role_id".($this->diafan->_user->role_id ? " IN (0,".$this->diafan->_user->role_id.")" : "=0")
					." AND pr.user_id".($this->diafan->_user->id ? " IN (0,".$this->diafan->_user->id.")" : "=0")
					." AND threshold <= ".$this->order_summ;
				$group = ", pr.price_id HAVING"
					. ( ! empty($_GET["pr1"]) ? " MIN(pr.price)>=".$pr1 : '')
					. ( ! empty($_GET["pr2"]) ? (! empty($_GET["pr1"]) ? " AND" : "")." MIN(pr.price)<=".$pr2 : '');
            }
			$result = DB::query("SELECT DISTINCT(p.id), p.type FROM {shop_param} as p "
							. " INNER JOIN {shop_param_category_rel} AS c ON p.id=c.element_id "
							. ($this->diafan->configmodules("cat") ? " AND (c.cat_id=%d OR c.cat_id=0)" : "")
							. " WHERE p.search='1' AND p.trash='0' ORDER BY p.sort ASC", $this->diafan->cat);
            while ($row = DB::fetch_array($result))
            {
                if (($row["type"] == 'text' || $row["type"] == 'textarea' || $row["type"] == 'editor') && ! empty($_GET["p" . $row["id"]]))
                {
                    $val = trim($this->diafan->get_param($_GET, "p" . $row["id"], '', 1));
                    $where_param .= "
					INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                            . " AND pe" . $row["id"] . ".trash='0' AND pe" . $row["id"] . ".value" . _LANG . " LIKE '%%%h%%'";
                    $values_param[] = $row["id"];
                    $values_param[] = $val;
                    $getnav .= '&p' . $row["id"] . '=' . $val;
                }
                elseif ($row["type"] == 'date' && ( ! empty($_GET["p" . $row["id"] . "_1"]) || ! empty($_GET["p" . $row["id"] . "_2"])))
                {
                    $where_param .= "
					INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                            . " AND pe" . $row["id"] . ".trash='0'";
                    $values_param[] = $row["id"];
                    if ( ! empty($_GET["p" . $row["id"] . "_1"]))
                    {
                        $where_param .= " AND pe" . $row["id"] . ".value>='%s'";
                        $values_param[] = $this->diafan->formate_in_date($_GET["p" . $row["id"] . "_1"]);
                        $getnav .= '&p' . $row["id"] . '_1=' . trim($this->diafan->get_param($_GET, "p" . $row["id"] . "_1", '', 1));
                    }
                    if ( ! empty($_GET["p" . $row["id"] . "_2"]))
                    {
                        $where_param .= " AND pe" . $row["id"] . ".value<='%s'";
                        $values_param[] = $this->diafan->formate_in_date($_GET["p" . $row["id"] . "_2"]);
                        $getnav .= '&p' . $row["id"] . '_2=' . trim($this->diafan->get_param($_GET, "p" . $row["id"] . "_2", '', 1));
                    }
                }
                elseif ($row["type"] == 'datetime' && ( ! empty($_GET["p" . $row["id"] . "_1"]) || ! empty($_GET["p" . $row["id"] . "_2"])))
                {
                    $where_param .= "
					INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                            . " AND pe" . $row["id"] . ".trash='0'";
                    $values_param[] = $row["id"];
                    if ( ! empty($_GET["p" . $row["id"] . "_1"]))
                    {
                        $where_param .= " AND pe" . $row["id"] . ".value>='%s'";
                        $values_param[] = $this->diafan->formate_in_datetime($_GET["p" . $row["id"] . "_1"]);
                        $getnav .= '&p' . $row["id"] . '_1=' . trim($this->diafan->get_param($_GET, "p" . $row["id"] . "_1", '', 1));
                    }
                    if ( ! empty($_GET["p" . $row["id"] . "_2"]))
                    {
                        $where_param .= " AND pe" . $row["id"] . ".value<='%s'";
                        $values_param[] = $this->diafan->formate_in_datetime($_GET["p" . $row["id"] . "_2"]);
                        $getnav .= '&p' . $row["id"] . '_2=' . trim($this->diafan->get_param($_GET, "p" . $row["id"] . "_2", '', 1));
                    }
                }
                elseif ($row["type"] == 'numtext' && ( ! empty($_GET["p" . $row["id"] . "_2"]) || ! empty($_GET["p" . $row["id"] . "_1"])))
                {
                    $val1 = (int) $this->diafan->get_param($_GET, "p" . $row["id"] . "_1", '', 2);
                    $val2 = (int) $this->diafan->get_param($_GET, "p" . $row["id"] . "_2", '', 2);
                    $where_param .= "
					INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                            . " AND pe" . $row["id"] . ".trash='0'"
                            . ($val1 ? " AND pe" . $row["id"] . ".value>=%d" : '')
                            . ($val2 ? " AND pe" . $row["id"] . ".value<=%d" : '')
                    ;
                    $values_param[] = $row["id"];
                    if ($val1)
                    {
                        $values_param[] = $val1;
                        $getnav .= '&p' . $row["id"] . '_1=' . $val1;
                    }
                    if ($val2)
                    {
                        $values_param[] = $val2;
                        $getnav .= '&p' . $row["id"] . '_2=' . $val2;
                    }
                }
                elseif ($row["type"] == 'checkbox' && ! empty($_GET["p" . $row["id"]]))
                {
                    $where_param .= "
					INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                            . " AND pe" . $row["id"] . ".trash='0' AND pe" . $row["id"] . ".value='1'";
                    $values_param[] = $row["id"];
                    $getnav .= '&p' . $row["id"] . '=1';
                }
                elseif (($row["type"] == 'select' || $row["type"] == 'multiple') && ! empty($_GET["p" . $row["id"]]))
                {
                    if ( ! is_array($_GET["p" . $row["id"]]))
                    {
                        $val = (int) $this->diafan->get_param($_GET, "p" . $row["id"], '', 2);
                        $where_param .= "
						INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                                . " AND pe" . $row["id"] . ".trash='0' AND pe" . $row["id"] . ".value='%d'";
                        $values_param[] = $row["id"];
                        $values_param[] = $val;
                        $getnav .= '&p' . $row["id"] . '=' . $val;
                    }
                    else
                    {
                        $vals = array();
                        foreach ($_GET["p" . $row["id"]] as $val)
                        {
                            if ($val)
                            {
                                $val = intval($val);
                                $vals[] = $val;
                                $getnav .= '&p' . $row["id"] . '[]=' . $val;
                            }
                        }
                        if ( ! empty($vals))
                        {
                            $where_param .= "
							INNER JOIN {shop_param_element} AS pe" . $row["id"] . " ON pe" . $row["id"] . ".element_id=s.id AND pe" . $row["id"] . ".param_id='%d'"
                                    . " AND pe" . $row["id"] . ".trash='0' AND pe" . $row["id"] . ".value IN (" . implode(", ", $vals) . ")";
                            $values_param[] = $row["id"];
                            if ( ! empty($_GET["pr1"]) || ! empty($_GET["pr2"]))
                            {
                                $where_param .= " AND (pr.param_id<>pe" . $row["id"] . ".param_id OR pr.param_value=pe" . $row["id"] . ".value)";
                            }
                        }
                    }
                }
            }
        }
		$values = array_merge($values_param, $values);
        return true;
    }

    /**
     * Генерирует ссылки на форму редактирования
     *
     * @param array $row данные о товаре
     * @return boolean true
     */
    private function useradmin(&$row)
    {
		if(! empty($row["price_arr"]))
		{
			foreach($row["price_arr"] as $i => $price)
			{
				if ( ! empty($row['discount']))
				{
					$row["price_arr"][$i]["old_price"] = $this->diafan->_useradmin->get($row["price_arr"][$i]["old_price"], 'price', $row["price_arr"][$i]["price_id"], 'shop_price', '', 'text');
				}
				else
				{
					$row["price_arr"][$i]["price"] = $this->diafan->_useradmin->get($row["price_arr"][$i]["price"], 'price', $row["price_arr"][$i]["price_id"], 'shop_price', '', 'text');
				}
			}
		}
		elseif(! empty($row["price"]))
		{
			if ( ! empty($row['discount']))
			{
				$row["old_price"] = $this->diafan->_useradmin->get($row["old_price"], 'price', $row["price_id"], 'shop_price', '', 'text');
			}
			else
			{
				$row["price"] = $this->diafan->_useradmin->get($row["price"], 'price', $row["price_id"], 'shop_price', '', 'text');
			}
		}

        if ( ! empty($row["name"]))
        {
            $row["name"] = $this->diafan->_useradmin->get($row["name"], 'name', $row["id"], 'shop', _LANG);
        }
        if ( ! empty($row["article"]))
        {
            $row["article"] = $this->diafan->_useradmin->get($row["article"], 'article', $row["id"], 'shop', '', 'text');
        }
        if ( ! empty($row["text"]))
        {
            $row["text"] = $this->diafan->_useradmin->get($row["text"], 'text', $row["id"], 'shop', _LANG);
        }

        if ( ! empty($row["anons"]))
        {
			$row["anons"] = $this->diafan->_useradmin->get($row["anons"], 'anons', $row["id"], 'shop', _LANG);
		}

        if ( ! empty($row["param"]))
        {
			foreach ($row["param"] as $k => $param)
			{
				$row["param"][$k]["name"] = $this->diafan->_useradmin->get($param["name"], 'name', $param["id"], 'shop_param');
				if ( ! empty($param["value_id"]))
				{
					$lang = in_array($param["type"], array('text', 'textarea', 'editor')) ? _LANG : '';
					$row["param"][$k]["value"] = $this->diafan->_useradmin->get($param["value"], 'value', $param["value_id"], 'shop_param_element', $lang, $param["type"]);
				}
			}
		}

        return true;
    }

    /**
     * Генерирует данные для страницы сравнения товаров
     *
     * @return boolean true
     */
    public function compare()
    {
        if (empty($_GET['ids']))
        {
            return false;
        }

        foreach ($_GET['ids'] as &$value)
        {
            $value = intval($value);
        }

        $result = DB::query("SELECT id, [name], cat_id, timeedit,  site_id, no_buy, article, is_file
		  FROM {shop} WHERE id IN (" . implode(',', $_GET['ids']) . ") AND site_id=%d AND
		  act='1' AND trash='0'", $this->diafan->cid);

        if ( ! (DB::num_rows($result) > 0))
        {
            return FALSE;
        }

        $this->result["rows"] = $this->get_elements($result, 'list', 'id');

        $existed_shop_ids = array();
        foreach ($this->result["rows"] as $value)
        {
            $existed_shop_ids[] = $value['id'];
        }

        $existed_params_result = DB::query("SELECT e.param_id as id, p.[name] FROM {shop_param_element}
          AS e INNER JOIN {shop_param} AS p ON e.param_id=p.id WHERE e.trash='0' AND p.trash='0' AND
          e.element_id IN(" . implode(', ', $existed_shop_ids) . ") GROUP BY e.param_id  ORDER BY p.sort");

        $this->result["existed_params"] = array();
        while ($row = DB::fetch_array($existed_params_result))
        {
            $this->result["existed_params"][] = $row;
        }

        $this->result["param_differences"] = $this->compare_get_param_difference($this->result["rows"], $this->result["existed_params"]);

		if($this->diafan->configmodules("theme_compare"))
		{
			$this->result["theme"] = $this->diafan->configmodules("theme_compare");
		}
		if($this->diafan->configmodules("view_compare"))
		{
			$this->result["view"] = $this->diafan->configmodules("view_compare");
		}
		else
		{
			$this->result["view"] = 'compare';
		}
        return $this->result;
    }

    /**
     * Находит отличающиеся характеристики у сравниваемых товаров
     *
	 * @param array $goods товары
	 * @param array $existed_params характеристики товаров
     * @return array
     */
    private function compare_get_param_difference(&$goods, &$existed_params)
    {
        if (empty($goods) || empty($existed_params))
        {
            return array();
        }
        $param_differences = array();
        $param_values = array();
        $existed_params_ids = array();

        foreach ($existed_params as $param)
        {
            $existed_params_ids[] = $param['id'];
        }

        foreach ($goods as $good)
        {
            $this->_compare_get_param_difference($good["param"], $param_differences, $param_values, $existed_params_ids);
        }

        return $param_differences;
    }

    /**
     * Проходит по всем характеристикам товара и находит отличащиеся от общих значения
     *
	 * @param array $params характеристики текущего товара
	 * @param array $param_differences отличающиеся характеристики, найденные до текущей итерации
	 * @param array $param_values значения характерстик, общие для всех товаров
	 * @param array $existed_params_ids все характеристики выбранных товаров
     * @return void
     */
    private function _compare_get_param_difference(&$params, &$param_differences, &$param_values, &$existed_params_ids)
    {
        $ids = array();

        foreach ($params as $param)
        {
            if (in_array($param['id'], $param_differences))
            {
                continue;
            }
            if (isset($param_values[$param['id']]) && $param_values[$param['id']] != $param["value"])
            {
                $param_differences[] = $param['id'];
                continue;
            }

            $param_values[$param['id']] = $param["value"];
            $ids[] = $param['id'];
        }

        foreach ($existed_params_ids as $id)
        {
            if ( ! in_array($id, $ids) && ! in_array($id, $param_differences))
            {
                $param_differences[] = $id;
            }
        }
    }

    /**
     * Отдает купленный товар-файл
	 * 
     * @return void
     */
    public function file_get()
    {
        $date = date('Y-m-d H:i');

        if ( ! $row = DB::fetch_array(DB::query("SELECT a.id, a.name, a.extension, a.size,
            a.module_name
            FROM {shop_files_codes} as c INNER JOIN {attachments} AS a 
            ON a.element_id=c.shop_id AND a.module_name='%s'
            WHERE code='%s' AND date_finish>='%s'", $this->diafan->module, $_GET["code"], $date)))
        {
            include ABSOLUTE_PATH . 'includes/404.php';
        }

        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
        header('Cache-Control: max-age=86400');
        if ($row["extension"])
        {
            header("Content-type: " . $row["extension"]);
        }

        if ($row["size"] > 0)
        {
            header("Content-length: " . $row["size"]);
        }

        header("Content-Disposition: attachment; filename=" . $row["name"]);
        header('Content-transfer-encoding: binary');
        header("Connection: close");

        $file_path = ABSOLUTE_PATH . USERFILES . "/" . $row["module_name"] . "/files/" . $row["id"];

        $handle = fopen($file_path, "r");

        echo fread($handle, $row["size"]);
        exit;
    }

    /**
     * Позволяет добавлять характеристики товара  для сортировки
	 * 
     * @return array $params_for_sort
     */
    private function expand_sort_with_params()
    {
        $sort_fields_names = array(1 => _LANG_PRICE, 3 => _LANG_NAME_GOOD);

        $sort_directions = array(
            1 => 'pr.price ASC',
            2 => 'pr.price DESC',
            3 => 's.name ASC',
            4 => 's.name DESC'
        );

        $param_ids = array();

        $result = DB::query("SELECT p.id, p.[name], p.type FROM {shop_param} AS p "
                        . " INNER JOIN {shop_param_category_rel} AS cr ON cr.element_id=p.id AND cr.trash='0' "
                        . ($this->diafan->cat ? " AND (cr.cat_id=%d OR cr.cat_id=0)" : "")
                        . " WHERE p.trash='0' AND p.display_in_sort='1' AND p.type IN
            ('text', 'numtext', 'date', 'datetime', 'checkbox') ORDER BY p.sort", $this->diafan->cat);

        while ($row = DB::fetch_array($result))
        {
            $sort_directions[] = ' sp.[value] ASC ';
            $param_ids[count($sort_directions)] = $row['id'];
            $sort_fields_names[count($sort_directions)] = $row['name'];

            $sort_directions[] = ' sp.[value] DESC ';
            $param_ids[count($sort_directions)] = $row['id'];
        }

        $use_params_for_sort = $this->diafan->sort > 4 ? TRUE : FALSE;

        return array('sort_fields_names' => $sort_fields_names, 'sort_directions' => $sort_directions,
            'param_ids' => $param_ids, 'use_params_for_sort' => $use_params_for_sort);
    }

    /**
     * Формирует список характеристик, которые могут быть выбраны для сортировки
	 * 
     * @return array
     */
    private function get_sort_links()
    {
        $result = array();

        foreach ($this->sort_config['sort_directions'] as $key => $value)
        {
            $result[$key] = $this->diafan->sort != $key ? $this->diafan->_route->current_link("", array('sort' => $key)) : '';
        }

        return $result;
    }
}